@@ -33,24 +33,45 @@
 #include <stdint.h>
 #include <stdlib.h>
 
-#if ARCH_X86_64 && defined(_WIN32)
-/* setjmp/longjmp on 64-bit Windows will try to use SEH to unwind the stack,
- * which doesn't work for assembly functions without unwind information. */
+#ifdef _WIN32
 #include <windows.h>
-#define checkasm_context CONTEXT
-#define checkasm_save_context() RtlCaptureContext(&checkasm_context_buf)
-#define checkasm_load_context() RtlRestoreContext(&checkasm_context_buf, NULL)
-#else
+#if ARCH_X86_32
 #include <setjmp.h>
-#define checkasm_context jmp_buf
+typedef jmp_buf checkasm_context;
 #define checkasm_save_context() setjmp(checkasm_context_buf)
 #define checkasm_load_context() longjmp(checkasm_context_buf, 1)
+#elif WINAPI_FAMILY_PARTITION(WINAPI_PARTITION_DESKTOP)
+/* setjmp/longjmp on Windows on architectures using SEH (all except x86_32)
+ * will try to use SEH to unwind the stack, which doesn't work for assembly
+ * functions without unwind information. */
+typedef struct { CONTEXT c; int status; } checkasm_context;
+#define checkasm_save_context() \
+    (checkasm_context_buf.status = 0, \
+     RtlCaptureContext(&checkasm_context_buf.c), \
+     checkasm_context_buf.status)
+#define checkasm_load_context() \
+    (checkasm_context_buf.status = 1, \
+     RtlRestoreContext(&checkasm_context_buf.c, NULL))
+#else
+typedef void* checkasm_context;
+#define checkasm_save_context() 0
+#define checkasm_load_context() do {} while (0)
+#endif
+#else
+#include <setjmp.h>
+typedef sigjmp_buf checkasm_context;
+#define checkasm_save_context() sigsetjmp(checkasm_context_buf, 1)
+#define checkasm_load_context() siglongjmp(checkasm_context_buf, 1)
 #endif
 
 #include "include/common/attributes.h"
 #include "include/common/bitdepth.h"
 #include "include/common/intops.h"
 
+#if ARCH_ARM
+#include "src/arm/arm-arch.h"
+#endif
+
 int xor128_rand(void);
 #define rnd xor128_rand
 
@@ -75,6 +96,7 @@ int checkasm_fail_func(const char *msg, ...);
 void checkasm_update_bench(int iterations, uint64_t cycles);
 void checkasm_report(const char *name, ...);
 void checkasm_set_signal_handler_state(int enabled);
+void checkasm_handle_signal(void);
 extern checkasm_context checkasm_context_buf;
 
 /* float compare utilities */
@@ -101,7 +123,7 @@ int float_near_abs_eps_array_ulp(const float *a, const float *b, float eps,
     declare_new(ret, __VA_ARGS__)\
     void *func_ref, *func_new;\
     typedef ret func_type(__VA_ARGS__);\
-    checkasm_save_context()
+    if (checkasm_save_context()) checkasm_handle_signal()
 
 /* Indicate that the current test has failed */
 #define fail() checkasm_fail_func("%s:%d", __FILE__, __LINE__)
@@ -128,6 +150,9 @@ static inline uint64_t readtime(void) {
 }
 #define readtime readtime
 #endif
+#elif CONFIG_MACOS_KPERF
+uint64_t checkasm_kperf_cycles(void);
+#define readtime() checkasm_kperf_cycles()
 #elif (ARCH_AARCH64 || ARCH_ARM) && defined(__APPLE__)
 #include <mach/mach_time.h>
 #define readtime() mach_absolute_time()
@@ -179,6 +204,31 @@ static inline uint64_t readtime(void) {
     return (((uint64_t)tbu) << 32) | (uint64_t)tbl;
 }
 #define readtime readtime
+#elif ARCH_RISCV
+#include <time.h>
+static inline uint64_t clock_gettime_nsec(void) {
+  struct timespec ts;
+  clock_gettime(CLOCK_MONOTONIC_RAW, &ts);
+  return ((uint64_t)ts.tv_sec*1000000000u) + (uint64_t)ts.tv_nsec;
+}
+#define readtime clock_gettime_nsec
+#elif ARCH_LOONGARCH
+static inline uint64_t readtime(void) {
+#if ARCH_LOONGARCH64
+    uint64_t a, id;
+    __asm__ __volatile__("rdtime.d  %0, %1"
+                         : "=r"(a), "=r"(id)
+                         :: );
+    return a;
+#else
+    uint32_t a, id;
+    __asm__ __volatile__("rdtimel.w  %0, %1"
+                         : "=r"(a), "=r"(id)
+                         :: );
+    return (uint64_t)a;
+#endif
+}
+#define readtime readtime
 #endif
 
 /* Verifies that clobbered callee-saved registers
@@ -208,7 +258,7 @@ void checkasm_simd_warmup(void);
  * handled orthogonally from integer parameters passed in GPR registers. */
 #define IGNORED_FP_ARGS 8
 #endif
-#ifdef HAVE_C11_GENERIC
+#if HAVE_C11_GENERIC
 #define clobber_type(arg) _Generic((void (*)(void*, arg))NULL,\
      void (*)(void*, int32_t ): clobber_mask |= 1 << mpos++,\
      void (*)(void*, uint32_t): clobber_mask |= 1 << mpos++,\
@@ -256,12 +306,12 @@ void checkasm_simd_warmup(void);
 /* Use a dummy argument, to offset the real parameters by 2, not only 1.
  * This makes sure that potential 8-byte-alignment of parameters is kept
  * the same even when the extra parameters have been removed. */
-void checkasm_checked_call_vfp(void *func, int dummy, ...);
+extern void (*checkasm_checked_call_ptr)(void *func, int dummy, ...);
 #define declare_new(ret, ...)\
     ret (*checked_call)(void *, int dummy, __VA_ARGS__,\
                         int, int, int, int, int, int, int, int,\
                         int, int, int, int, int, int, int) =\
-    (void *)checkasm_checked_call_vfp;
+    (void *)checkasm_checked_call_ptr;
 #define call_new(...)\
     (checkasm_set_signal_handler_state(1),\
      checked_call(func_new, 0, __VA_ARGS__, 11, 10, 9, 8, 7, 6, 5, 4, 3, 2, 1, 0, 0, 0, 0));\
@@ -283,6 +333,28 @@ void checkasm_stack_clobber(uint64_t clobber, ...);
      checked_call(func_new, 0, 0, 0, 0, 0, 0, 0, __VA_ARGS__,\
                   7, 6, 5, 4, 3, 2, 1, 0, 0, 0, 0, 0, 0, 0, 0));\
     checkasm_set_signal_handler_state(0)
+#elif ARCH_RISCV
+#define declare_new(ret, ...)\
+    ret (*checked_call)(void *, int, int, int, int, int, int, int,\
+                        __VA_ARGS__, int, int, int, int, int, int, int, int,\
+                        int, int, int, int, int, int, int) =\
+    (void *)checkasm_checked_call;
+#define call_new(...)\
+    (checkasm_set_signal_handler_state(1),\
+     checked_call(func_new, 0, 0, 0, 0, 0, 0, 0, __VA_ARGS__,\
+                  7, 6, 5, 4, 3, 2, 1, 0, 0, 0, 0, 0, 0, 0, 0));\
+    checkasm_set_signal_handler_state(0)
+#elif ARCH_LOONGARCH
+#define declare_new(ret, ...)\
+    ret (*checked_call)(void *, int, int, int, int, int, int, int,\
+                        __VA_ARGS__, int, int, int, int, int, int, int, int,\
+                        int, int, int, int, int, int, int) =\
+    (void *)checkasm_checked_call;
+#define call_new(...)\
+    (checkasm_set_signal_handler_state(1),\
+     checked_call(func_new, 0, 0, 0, 0, 0, 0, 0, __VA_ARGS__,\
+                  7, 6, 5, 4, 3, 2, 1, 0, 0, 0, 0, 0, 0, 0, 0));\
+    checkasm_set_signal_handler_state(0)
 #else
 #define declare_new(ret, ...)
 #define call_new(...)\
